package watch

import (
	"crypto/md5"
	"encoding/hex"
	"io"
	"os"
	"sync"
	"time"
)

type FileHash struct {
	Name      string
	Hash      string
	expiresAt int64
}

var (
	fileHashCache map[string]FileHash
	fileHashMutex *sync.Mutex
)

func init() {
	fileHashCache = make(map[string]FileHash)
	fileHashMutex = &sync.Mutex{}

	go func() {
		for {
			time.Sleep(time.Minute * 60 * 24) // 1 day
			fileHashMutex.Lock()
			for k, v := range fileHashCache {
				if v.expiresAt < time.Now().Unix() {
					delete(fileHashCache, k)
				}
			}
			fileHashMutex.Unlock()
		}
	}()

}

func RemoveFileHash(path string) {
	fileHashMutex.Lock()
	defer fileHashMutex.Unlock()

	if _, ok := fileHashCache[path]; ok {
		delete(fileHashCache, path)
	}

}

func GetFileHash(path string) string {
	fileHashMutex.Lock()
	defer fileHashMutex.Unlock()

	if hash, ok := fileHashCache[path]; ok {
		return hash.Hash
	}

	h, err := buildHash(path)
	if err != nil {
		return ""
	}

	fileHashCache[path] = h

	return h.Hash
}

type FileHashes map[string]FileHash

func buildHash(path string) (FileHash, error) {
	var h FileHash // HASH

	h.Name = path
	h.Hash, _ = hashFile(path)
	h.expiresAt = time.Now().Unix() + 3600*24 // 1 day
	return h, nil
}

func hashFile(path string) (string, error) {
	var h string // HASH

	file, err := os.Open(path)
	if err != nil {
		return h, err
	}

	defer file.Close()

	hash := md5.New()

	if _, err := io.Copy(hash, file); err != nil {
		return h, err
	}

	hashInBytes := hash.Sum(nil)[:16]
	h = hex.EncodeToString(hashInBytes)

	return h, nil

}
